

/********************************************************************************
uses Adafruit SPI library for 1.8 TFT SPI display with ST7735R chip
  This library works with the Adafruit 1.8" TFT Breakout w/SD card
  ----> http://www.adafruit.com/products/358
  as well as Adafruit raw 1.8" TFT display
  ----> http://www.adafruit.com/products/618



code for fan temp credits: 
Crenn from http://thebestcasescenario.com
project by Charles Gantt from http://themakersworkbench.com
To disable interrupts: cli(); disable global interrupts and
to enable them: sei(); enable interrupts
http://www.themakersworkbench.com/content/tutorial/reading-pc-fan-rpm-arduino

*************************************************************************************/

//LCD pins and libraries
#define sclk 13
#define mosi 11
#define cs   10
#define dc   9
#define rst  8  // you can also connect this to the Arduino reset
#define SD_CS    4  // Chip select line for SD card
#define fanSpdPin 6       // the pin where fan is
#define tempPin A1
#define pushBotton 3  // LED Light control pusbutton 
#define LEDpin 5      // Pin to control PWM to LEDs
#define hallsensor 2 //The pin location of the sensor - yellow wire for fan speed sensor
#include <Adafruit_GFX.h>    // Core graphics library
#include <Adafruit_ST7735.h> // Hardware-specific library
#include <SPI.h>
#include <SD.h>
#if defined(__SAM3X8E__)
    #undef __FlashStringHelper::F(string_literal)
    #define F(string_literal) string_literal
#endif
#define BUFFPIXEL 20  //for BmpDraw

// Option 1: use any pins but a little slower
//Adafruit_ST7735 tft = Adafruit_ST7735(cs, dc, mosi, sclk, rst);

// Option 2: must use the hardware SPI pins
// (for UNO thats sclk = 13 and sid = 11) and pin 10 must be
// an output. This is much faster - also required if you want
// to use the microSD card (see the image drawing example)
Adafruit_ST7735 tft = Adafruit_ST7735(cs, dc, rst);

//END LCD Pins and Libraries


int NbTopsFan; 
int Calc;
//int fanSpdPin = 6;       // the pin where fan is
//int tempPin = A1;
int temp;
int tempMin = 30;   // the temperature to start the fan
int tempMax = 70;   // the maximum temperature when fan is at 100%
int lastVal = 0;
int lastRPM = 0;
//int pushBotton = 3;  // LED Light control pusbutton 
//int LEDpin = 5;      // Pin to control PWM to LEDs
int level = 0;       // LED light level selected
int levelLEDGraphic = 1;  
int buttonState;             // LED pushbutton - the current reading from the input pin
int lastButtonState = LOW;   // LED:  the previous reading from the input pin
long lastDebounceTime = 0;  // LED pushbotton debounce control - the last time the output pin was toggled
long debounceDelay = 150;    // LED pushbotton debounce control - the debounce time; increase if the output flickers
boolean fanX = false;        //used to flip fan animation from X to +
char buf[3];  //for drawing circles - temperature bar
int avgFanSpeed;
int avgCount=1;
//int hallsensor = 2; //The pin location of the sensor - yellow wire for fan speed sensor

//Defines the structure for multiple fans and 
//their dividers 
typedef struct{char fantype;  unsigned int fandiv; }fanspec;
//Definitions of the fans
//This is the varible used to select the fan and it's divider,
//set 1 for unipole hall effect sensor
//and 2 for bipole hall effect sensor
fanspec fanspace[3]={{0,1},{1,2},{2,8}}; char fan = 1;

//i,j
int fanSpinMatrix[4][2] = {
 {9,8},  //top lf
 {17,8}, //top rt
 {17,18}, //bot rt
 {9,18}   //bot lf
};
int i=0,j=0;

 
//This is the setup function where the serial port is initialised,
//and the interrupt is attached
void setup()
{ 
  pinMode(LEDpin, OUTPUT);
  analogWrite(LEDpin,255);  //ensure the LED is off at startup.
  pinMode(hallsensor, INPUT);
  pinMode(fanSpdPin, OUTPUT);
  pinMode(tempPin, INPUT);
  Serial.begin(9600);
  attachInterrupt(0, rpm, RISING);  //for Fan RPM monitoring.  Nano - D2
  analogWrite(fanSpdPin, 0);  // spin the fan at the fanSpeed speed

  attachInterrupt(1, monitorPushbutton, CHANGE);  //for LED light control switch monitoring.  Nano - D3

  //LCD Init
  // If your TFT's plastic wrap has a Black Tab, use the following:
  //tft.initR(INITR_BLACKTAB);   // initialize a ST7735S chip, black tab
  // If your TFT's plastic wrap has a Red Tab, use the following:
  tft.initR(INITR_REDTAB);   // initialize a ST7735R chip, red tab
  // If your TFT's plastic wrap has a Green Tab, use the following:
  //tft.initR(INITR_GREENTAB); // initialize a ST7735R chip, green tab

  //set to landscape
  tft.setRotation(tft.getRotation()-1);    
  //END LCD Init
  

  //display splash logo
 //Serial.print("Initializing SD card...");
  if (!SD.begin(SD_CS)) {
    //Serial.println("failed!");
    return;
  }
  //Serial.println("OK!");
  bmpDraw("LulzBot2.bmp", 0, 0);
  delay(3000);
  tft.fillScreen(ST7735_BLACK);
  
  //Draw white rectangles
  //tft.drawRoundRect(x, y, width, ht, 4, ST7735_WHITE);
  //outer bounding box
  //tft.drawRoundRect(0, 1, 158, 125, 4, ST7735_WHITE);
  //Rambo bounding box
  tft.drawRoundRect(0, 1, 158, 40, 4, ST7735_WHITE);

  //Temperature bounding box
  tft.drawRoundRect(0, 40, 158, 40, 4, ST7735_WHITE);

//Temperature bounding box
  tft.drawRoundRect(0, 79, 158, 46, 4, ST7735_WHITE);


//  tft.setTextColor(ST7735_RED); 
//  tft.setTextSize(2);
  
  //print temp label
  //tft.setTextSize(1);   
  //tft.setCursor(3, 10);
  //tft.print("Case Temp: "); 

  //Setup Temp area
  //draw the rambo text and rectangle
  tft.setTextSize(1);
  tft.setCursor(10, 10);
  tft.setTextColor(ST7735_WHITE);
  tft.print("RAMBO");
  tft.drawRoundRect(6, 7, 38, 13, 3, ST7735_MAGENTA);

  //draw the temp bar outer area
  tft.drawRoundRect(6, 25, 70, 10, 3, ST7735_BLUE);
  
  //place the C E N T textt
  tft.setTextColor(ST7735_GREEN);
  tft.setCursor(146, 8);
  tft.print("C");
  tft.setCursor(146, 18);
  tft.print("E");
  tft.setCursor(146, 28);
  tft.print("L");
  
  //setup Fan graphics
  tft.setTextSize(1);
  tft.setCursor(10, 56);
  tft.setTextColor(ST7735_WHITE);
  tft.print("FAN");
  tft.drawRoundRect(6, 53, 27, 13, 3, ST7735_MAGENTA);  
  doFanStaticGraphic(49,47);  //draw the non-movable parts of the fan.
  //do RPM Text to right of fan RPM result
  tft.setTextColor(ST7735_GREEN);
  tft.setCursor(146, 46);
  tft.print("R");
  tft.setCursor(146, 56);
  tft.print("P");
  tft.setCursor(146, 66);
  tft.print("M");
  

  //setup LED graphics
  //draw the outer rectangle
  tft.drawRoundRect(6, 88, 27, 13, 3, ST7735_MAGENTA);  
  tft.setTextSize(1);
  tft.setCursor(10, 91);
  tft.setTextColor(ST7735_WHITE);
  tft.print("LED");
  //draw the LED bar outer area
  tft.drawRoundRect(6, 106, 70, 10, 3, ST7735_BLUE);

  
  
  //setup LED info
  //tft.setTextSize(2);
  //tft.setTextColor(ST7735_RED);
  //print LED label
  //tft.setCursor(3, 90);  
  //tft.print("LED: ");        

  //draw the inital LED icon
  //bmpDraw("LEDOff.bmp", 53, 88);

  //draw the MINI text   bmp.bmp, x,y
  bmpDraw("MINI.bmp", 90, 95); 

}



void loop ()
//Set NbTops to 0 ready for calculations
{ 
  //monitorPushbutton();  - disable - using interrupt.

  // Calculate the Fan speed
  NbTopsFan = 0;
  //Enables interrupts
  sei();
  //Wait 1 second
  delay (1000);  //was 1000  if 1000 below calc uses * 60
  //Disable interrupts
  cli();
  //Times NbTopsFan (which is apprioxiamately the fequency the fan 
  //is spinning at) by 60 seconds before dividing by the fan's divider
  Calc = ((NbTopsFan * 60)/fanspace[fan].fandiv);
  // END Calculate the Fan speed
  
  
  //Temperature
  temp = readTemp();     // get the temperature
  //doTempBar(temp, xloc, yloc, length, hieght, corner);
  doTempBar(temp, 6, 25, 70, 10, 3);
  //prints the Case Temp value in red
  printVal(90,14, 2, temp, lastVal, ST7735_GREEN);
  lastVal = temp;

  //Temperature: Manage the Fan Speed
  //map(temp, lowTemp, maxTemp,fanMinSpeed, FanMaxSpeed
  analogWrite(fanSpdPin, map(temp, 21, 80, 50, 255));
  
    
  //Fan RPM animated graphic
  doAnimateFan(49,47);

  //show fan speed value
  dispFanSpeed();
    
    
 
} 

//##############################################################################################

int readTemp() {  // get the temperature and convert it to celsius
  temp = analogRead(tempPin);
  return temp * 0.48828125;
}

void dispFanSpeed()
{
  avgFanSpeed = avgFanSpeed + Calc;
  if (avgCount == 10)
  {
    avgFanSpeed = avgFanSpeed / 10;
    //Fan RPM value in green
    if (avgFanSpeed > 0 && avgFanSpeed < 10000)
    {
      //printVal(90,54, 2, Calc, lastRPM, ST7735_GREEN);
      //lastRPM = Calc;
      printVal(90,54, 2, avgFanSpeed, lastRPM, ST7735_GREEN);
      lastRPM = avgFanSpeed;
    }
    avgCount=1;
    avgFanSpeed=0;
  }
  avgCount++;  
}

void testdrawtext(char *text, uint16_t color) {
  tft.setCursor(0, 0);
  tft.setTextColor(color);
  tft.setTextWrap(true);
  tft.print(text);
}

void doTempBar(int tmp, int x, int y, int w, int h, int corner)
  {
    //MAP y = map(x, fromLow, fromHigh, toLow, toHigh); mintemp, maxtemp, bar 0, bar width minus 2
    tmp = map(tmp, 0, 100, 0, w - 2);
    // tft.drawRoundRect(x, y, w, h, cornerradius, color);
    
    //tft.drawRoundRect(15, 70, 120, 10, 3, ST7735_WHITE);
    //tft.fillRect(x,y,width, height,colour)
    //clear out prev values
    tft.fillRect(x + 2, y + 3, 65, corner + 2, ST7735_BLACK);
    //show the new value as a bar
    tft.fillRect(x + 2, y + 3, tmp, corner + 2, ST7735_RED);  
    
    //draw the degree symbol after the number - this will auto adjust based on the number size
    tft.drawCircle(95 + (11 * strlen(itoa(temp, buf, 10))), 14, 2, ST7735_GREEN);    
  }


void doLEDBar(int tmp, int x, int y, int w, int h, int corner)
  {
    //MAP y = map(x, fromLow, fromHigh, toLow, toHigh); mintemp, maxtemp, bar 0, bar width minus 2
    tmp = map(tmp, 0, 100, 0, w - 2);
    //tft.fillRect(x,y,width, height,colour)
    //clear out prev values
    tft.fillRect(x + 2, y + 3, w-1, corner + 2, ST7735_BLACK);
    //show the new value as a bar
    tft.fillRect(x + 2, y + 3, tmp, corner + 2, ST7735_GREEN);  
    
  }


void printVal(int xLoc, int yLoc, int txtSize, int newV, int lastV, uint16_t color)
{
  tft.setTextSize(txtSize);
  tft.setCursor(xLoc, yLoc);
  tft.setTextColor(ST7735_BLACK); 
  tft.print(lastV);
  
  tft.setTextColor(color);
  tft.setCursor(xLoc, yLoc);
  tft.print(newV);
}


void doFanStaticGraphic(int x, int y)
{
    tft.drawRoundRect(x, y, 27, 27, 3, ST7735_BLUE);
    //tft.drawCircle(x, y, radius, color);
    tft.drawCircle(x+13, y+13, 9, ST7735_BLUE);
    tft.drawPixel(x+4, y+3, ST7735_GREEN);
    tft.drawPixel(x+22, y+3, ST7735_GREEN);
    tft.drawPixel(x+4, y+22, ST7735_GREEN);
    tft.drawPixel(x+22, y+22, ST7735_GREEN);
}


void doAnimateFan(int x, int y)
{

      if (fanX)
      {
       //tft.fillRect(x,y,width, height,colour)
        tft.fillRect(x+7, y+12, 13, 3, ST7735_MAGENTA);  
        tft.fillRect(x+12, y+7, 3, 13, ST7735_MAGENTA);  
      }
      else
      {
       //tft.fillRect(x,y,width, height,colour)
        tft.fillRect(x+7, y+12, 13, 3, ST7735_BLACK);  
        tft.fillRect(x+12, y+7, 3, 13, ST7735_GREEN);  
      }
      fanX = !fanX;
}


void monitorPushbutton()
{
            
          int reading = digitalRead(pushBotton);
          // If the switch changed, due to noise or pressing:
          if (buttonState != lastButtonState) {
          // reset the debouncing timer
            lastDebounceTime = millis();
          }         
          
         if ((millis() - lastDebounceTime) > debounceDelay) {
            // whatever the reading is at, it's been there for longer
            // than the debounce delay, so take it as the actual current state:
        
            // if the button state has changed:
            if (reading != buttonState) {
              buttonState = reading;
        
              // only toggle the LED if the new button state is HIGH
              if (buttonState == HIGH) {
                //ledState = !ledState;
                changeLevel();
                analogWrite(LEDpin, map(level, 0, 100, 255, 0));
              }
            }
          }
      delay(100);

  lastButtonState = reading;

}


void changeLevel()
{
  if (level <= 3)
  {
    level = 100;
  }
  else
  {
    level-=33;
    if (level < 3) level = 0;
  }

  //tft.setCursor(60, 90);
  //tft.print(levelLEDGraphic);
  //void doLEDBar(int tmp, int x, int y, int w, int h, int corner)
  doLEDBar(level, 6, 106, 68, 10, 3);

  /*
  switch (levelLEDGraphic)
  {
    case 1:
      bmpDraw("LEDHigh.bmp", 53, 88);
      break;
    case 2:
      bmpDraw("LEDMed.bmp", 53, 88);
      break;
    case 3:
      bmpDraw("LEDLow.bmp", 53, 88);
      break;
    case 4:
      bmpDraw("LEDOff.bmp", 53, 88);
      break;
    default: 
      // if nothing else matches, do the default
      bmpDraw("LEDBlank.bmp", 53, 88);
    break;  
  }
  levelLEDGraphic++;
  if (levelLEDGraphic ==5){levelLEDGraphic = 1;}
  */

}         
void bmpDraw(char *filename, uint8_t x, uint8_t y) {

  File     bmpFile;
  int      bmpWidth, bmpHeight;   // W+H in pixels
  uint8_t  bmpDepth;              // Bit depth (currently must be 24)
  uint32_t bmpImageoffset;        // Start of image data in file
  uint32_t rowSize;               // Not always = bmpWidth; may have padding
  uint8_t  sdbuffer[3*BUFFPIXEL]; // pixel buffer (R+G+B per pixel)
  uint8_t  buffidx = sizeof(sdbuffer); // Current position in sdbuffer
  boolean  goodBmp = false;       // Set to true on valid header parse
  boolean  flip    = true;        // BMP is stored bottom-to-top
  int      w, h, row, col;
  uint8_t  r, g, b;
  uint32_t pos = 0, startTime = millis();

  if((x >= tft.width()) || (y >= tft.height())) return;

  //Serial.println();
  //Serial.print("Loading image '");
  //Serial.print(filename);
  //Serial.println('\'');

  // Open requested file on SD card
  if ((bmpFile = SD.open(filename)) == NULL) {
    //Serial.print("File not found");
    return;
  }

  // Parse BMP header
  if(read16(bmpFile) == 0x4D42) { // BMP signature
    //Serial.print("File size: "); 
    //Serial.println(read32(bmpFile));
    read32(bmpFile);  //needs to be done if above Serial print is commented.
    (void)read32(bmpFile); // Read & ignore creator bytes
    bmpImageoffset = read32(bmpFile); // Start of image data
    //Serial.print("Image Offset: "); Serial.println(bmpImageoffset, DEC);
    // Read DIB header
    //Serial.print("Header size: "); 
    //Serial.println(read32(bmpFile));
    read32(bmpFile);
    bmpWidth  = read32(bmpFile);
    bmpHeight = read32(bmpFile);
    if(read16(bmpFile) == 1) { // # planes -- must be '1'
      bmpDepth = read16(bmpFile); // bits per pixel
      //Serial.print("Bit Depth: "); Serial.println(bmpDepth);
      if((bmpDepth == 24) && (read32(bmpFile) == 0)) { // 0 = uncompressed

        goodBmp = true; // Supported BMP format -- proceed!
        //Serial.print("Image size: ");
        //Serial.print(bmpWidth);
        //Serial.print('x');
        //Serial.println(bmpHeight);

        // BMP rows are padded (if needed) to 4-byte boundary
        rowSize = (bmpWidth * 3 + 3) & ~3;

        // If bmpHeight is negative, image is in top-down order.
        // This is not canon but has been observed in the wild.
        if(bmpHeight < 0) {
          bmpHeight = -bmpHeight;
          flip      = false;
        }

        // Crop area to be loaded
        w = bmpWidth;
        h = bmpHeight;
        if((x+w-1) >= tft.width())  w = tft.width()  - x;
        if((y+h-1) >= tft.height()) h = tft.height() - y;

        // Set TFT address window to clipped image bounds
        tft.setAddrWindow(x, y, x+w-1, y+h-1);

        for (row=0; row<h; row++) { // For each scanline...

          // Seek to start of scan line.  It might seem labor-
          // intensive to be doing this on every line, but this
          // method covers a lot of gritty details like cropping
          // and scanline padding.  Also, the seek only takes
          // place if the file position actually needs to change
          // (avoids a lot of cluster math in SD library).
          if(flip) // Bitmap is stored bottom-to-top order (normal BMP)
            pos = bmpImageoffset + (bmpHeight - 1 - row) * rowSize;
          else     // Bitmap is stored top-to-bottom
            pos = bmpImageoffset + row * rowSize;
          if(bmpFile.position() != pos) { // Need seek?
            bmpFile.seek(pos);
            buffidx = sizeof(sdbuffer); // Force buffer reload
          }

          for (col=0; col<w; col++) { // For each pixel...
            // Time to read more pixel data?
            if (buffidx >= sizeof(sdbuffer)) { // Indeed
              bmpFile.read(sdbuffer, sizeof(sdbuffer));
              buffidx = 0; // Set index to beginning
            }

            // Convert pixel from BMP to TFT format, push to display
            b = sdbuffer[buffidx++];
            g = sdbuffer[buffidx++];
            r = sdbuffer[buffidx++];
            tft.pushColor(tft.Color565(r,g,b));
          } // end pixel
        } // end scanline
        //Serial.print("Loaded in ");
        //Serial.print(millis() - startTime);
        //Serial.println(" ms");
      } // end goodBmp
    }
  }

  bmpFile.close();
  //if(!goodBmp) Serial.println("BMP format not recognized.");
}

// These read 16- and 32-bit types from the SD card file.
// BMP data is stored little-endian, Arduino is little-endian too.
// May need to reverse subscript order if porting elsewhere.

uint16_t read16(File f) {
  uint16_t result;
  ((uint8_t *)&result)[0] = f.read(); // LSB
  ((uint8_t *)&result)[1] = f.read(); // MSB
  return result;
}

uint32_t read32(File f) {
  uint32_t result;
  ((uint8_t *)&result)[0] = f.read(); // LSB
  ((uint8_t *)&result)[1] = f.read();
  ((uint8_t *)&result)[2] = f.read();
  ((uint8_t *)&result)[3] = f.read(); // MSB
  return result;
}

 
void rpm ()
//This is the function that the interupt calls
{ NbTopsFan++; }

